import os, pygame
from pygame.locals import *

from lights import Lamp
from obstacles import *
from warps import Teleporter
from enemies import *
from projectiles import *
from extras import HealthPack

#functions to create our resources
def load_image(name, colorkey=None):
    fullname = os.path.join('Data', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()

class Level:
    def __init__(self, level, screen, shadow):
        self.sprLights =        pygame.sprite.RenderPlain()
        self.levelLights =      []      # used for AI
        self.sprEnemies =       pygame.sprite.RenderPlain()
        self.sprObstacles =     pygame.sprite.RenderPlain()
        self.boxCount =         0 # how many boxes to press in this level?
        self.sprWarps =         pygame.sprite.RenderPlain()
        self.levelExtra =       []
        self.floorLights =      [0,0,0,0,0,0,0] # also used for AI
        self.loadLevel(level, screen, shadow)
        self.displayLevel(screen)
        self.levelComplete =    False   # level is not done yet
        self.sprExtra1 =        pygame.sprite.RenderPlain() # maybe change this?        
        self.sprEffects =       pygame.sprite.RenderPlain() # maybe change this?
        self.sprBullets =       pygame.sprite.RenderPlain() # maybe change this!
        self.sprSwords =        pygame.sprite.RenderPlain() # maybe change this?

    def loadLevel(self, level, screen, shadow):

        floor, null = load_image('bg1.gif') # change to loaded by map
        for floors in range(0, 7):
            screen.blit(floor, (0, floors*64+34))
        
        fullname = os.path.join('Levels', level)
        openLevel = open(fullname, 'r')
        levelLights =       []
        levelFloors =       []
        levelObstacles =    []
        levelEnemies =      []
        levelWarps  =       []
        levelExtra =       []
        for i in range(0, 6):
            line = openLevel.readline().strip() # thanks evilmrhenry for the fix
            if line == "Room":
                for j in range(0, 7):
                    levelFloors.append(openLevel.readline())
            elif line == "Lights":
                for j in range(0, 7):
                    self.levelLights.append(openLevel.readline())
            elif line == "Enemies":
                for j in range(0, 7):
                    levelEnemies.append(openLevel.readline())
            elif line == "Obstacles":
                for j in range(0, 7):
                    levelObstacles.append(openLevel.readline())
            elif line == "Warps":
                for j in range(0, 7):
                    levelWarps.append(openLevel.readline())
            elif line == "Extra":
                for j in range(0, 7):
                    levelExtra.append(openLevel.readline())
        self.loadFloors(levelFloors, screen)
        self.loadLights(self.levelLights, shadow)
        self.loadEnemies(levelEnemies)
        self.loadObstacles(levelObstacles)
        self.loadWarps(levelWarps)
        self.loadExtras(levelExtra)
        openLevel.close()

    def loadFloors(self, levelFloors, screen):
        for floors in range(0, 7):
            floorNum = int(levelFloors[floors])
            floorName =  "bg"+str(floorNum)+".gif"
            floorImage, floorRect = load_image(floorName)
            screen.blit(floorImage, (0, floors*64+34))

    def loadLights(self, levelLights, shadow):
        for floors in range(0, 7):
            floorLights = levelLights[floors]
            for lights in range(0, 5):
                if int(floorLights[lights]) != 0:
                    self.sprLights.add(Lamp((((130*lights+int(floorLights[lights])*10)-65),floors*64+34)))
        self.sprLights.draw(shadow)

    def loadEnemies(self, levelEnemies):
        for floors in range(0, 7):
            floorEnemies = levelEnemies[floors]
            for enemies in range(0, 5):
                if floorEnemies[enemies] != '0':
                    if floorEnemies[enemies] == 'A':
                        self.sprEnemies.add(AlienEnemy((128*enemies, floors*64+43), floors))
                    elif floorEnemies[enemies] == 'D':
                        self.sprEnemies.add(DogEnemy((128*enemies, floors*64+60), floors, False))
                    elif floorEnemies[enemies] == 'R':
                        self.sprEnemies.add(DogEnemy((128*enemies, floors*64+60), floors, True))

    def loadObstacles(self, levelObstacles):
        for floors in range(0, 7):
            floorObstacles = levelObstacles[floors]
            for obstacles in range(0, 5):
                if floorObstacles[obstacles] != '0':
                    if floorObstacles[obstacles] == 'c':
                        self.sprObstacles.add(ControlBox(((128*obstacles), floors*64+43)))
                        self.boxCount = self.boxCount + 1 # we have more boxes
                    elif floorObstacles[obstacles] == 'l':
                        self.sprObstacles.add(Ladder(((128*obstacles), floors*64+36), True))
                    elif floorObstacles[obstacles] == 'b':
                        self.sprObstacles.add(Ladder(((128*obstacles), floors*64+36), False))
                    elif floorObstacles[obstacles] == 'e':
                        self.sprObstacles.add(Exit(((128*obstacles), floors*64+43)))

    def loadWarps(self, levelWarps):
        """Designed to load our warps into the appropriate sprites """
        currentChar = 'A'
        xy = (0,0)
        ab = (0,0)
        for floors in range(0, 7):
            floorWarps = levelWarps[floors]
            for warps in range(0, 5):
                if floorWarps[warps] == currentChar:
                    xy = ((128*warps+23),(floors*64+42))
                    for floorsT in range(floors, 7):
                        floorWarpsT = levelWarps[floorsT]
                        for warpsT in range(0, 5):
                            if floorWarpsT[warpsT] == currentChar:
                                ab = ((128*warpsT+23),(floorsT*64+42))
                    self.sprWarps.add(Teleporter(xy, ab, currentChar, floors))
                    self.sprWarps.add(Teleporter(ab, xy, currentChar, floors))
                    currentChar = chr(ord(currentChar)+1) # ord returns the integer equivalent of char

    def loadExtras(self, levelExtras):
        for floors in range(0, 7):
            floorExtras = levelExtras[floors]
            for extras in range(0, 5):
                if floorExtras[extras] != '0':
                    if floorExtras[extras] == 'H':
                        print "Is not working for some reason?"
                        #self.sprExtra1.add(HealthPack(((128*obstacles), floors*64+43)))

    def displayLevel(self, screen):
        self.sprWarps.draw(screen)
        self.sprObstacles.draw(screen)

    def checkMap(self, player, screen, shadow):
        for obstacles in pygame.sprite.spritecollide(player, self.sprObstacles, 0):
            action = (obstacles.name(), obstacles.activate())
            if action[0] == "controlBox":
                x,y = action[1] # coordinates of the control box
                floor = (y-42)/64
                if self.floorLights[floor] == 0:
                    self.floorLights[floor] = 1 # set current floors lights to 1
                    floorLight = pygame.Surface((640,64))
                    floorLight = floorLight.convert()
                    floorLight.fill((0, 255, 0))
                    shadow.blit(floorLight, (0, 64*floor+34))
                    self.sprLights.draw(shadow)
                self.boxCount = self.boxCount - 1
                if (self.boxCount == 0):
                    for items in self.sprObstacles:
                        if items.name() == "myExit":
                            items.Complete()
                            break
                self.displayLevel(screen)
            if action[0] == "Complete" and self.boxCount == 0:
                self.levelComplete = True
            return action
        for warp in pygame.sprite.spritecollide(player, self.sprWarps, 0):
            action = ("warp", warp.teleportTo())
            return action
        return ("nothing", 0)

    def checkMapDown(self, player):  # used for ladders below
        px,py = player.rect.topleft
        py = py + 64
        player.rect.topleft = (px,py)
        py = py - 64
        for obstacles in pygame.sprite.spritecollide(player, self.sprObstacles, 0):
            if obstacles.name() == "ladder":
                action = ("downLadder", obstacles.activate())
                player.rect.topleft = (px,py)
                return action
        player.rect.topleft = (px,py)
        return ("nothing", 0)

    def playerAttack(self, playerAttack, player, playerInfo):
        location, name = playerAttack
        if (name == "shootLeft"):
            self.sprEffects.add(Flash(location, name))
            potentialList = []      # fill with potential bad guys hit
            enemyHit      = 0       # place enemy hit here
            x,y = location
            for enemies in self.sprEnemies:
                if enemies.level == ((y - 42) / 64):
                    px,py = enemies.rect.topleft
                    if px < x:
                        potentialList.append(enemies)
            if potentialList:
                px,py = potentialList[0].rect.topleft
                enemyHit = potentialList[0]
                closestX = x - px
                for potentials in potentialList:
                    tempX, null = potentials.rect.topleft
                    if (x - tempX) < closestX:
                        closestX = x - tempX
                        enemyHit = potentials
                playerInfo.setScore(player.addScore(enemyHit.hit(50, self.sprEnemies)))
                self.sprEffects.add(HitFlash((enemyHit.rect.center)))
        elif  (name == "shootRight"):
            self.sprEffects.add(Flash(location, name))
            potentialList = []      # fill with potential bad guys hit
            enemyHit      = 0       # place enemy hit here
            x,y = location
            for enemies in self.sprEnemies:
                if enemies.level == ((y - 42) / 64):
                    px,py = enemies.rect.topleft
                    if px > x:
                        potentialList.append(enemies)
            if potentialList:
                px,py = potentialList[0].rect.topleft
                enemyHit = potentialList[0]
                closestX = px - x
                for potentials in potentialList:
                    tempX, null = potentials.rect.topleft
                    if (tempX - x) < closestX:
                        closestX = tempX - x
                        enemyHit = potentials
                playerInfo.setScore(player.addScore(enemyHit.hit(50, self.sprEnemies)))
                self.sprEffects.add(HitFlash((enemyHit.rect.center)))
        elif (name == "swordLeft") or (name == "swordRight"):
            self.sprSwords.add(Sword(location, name))

    def update(self, screen, player, playerInfo):
        for enemies in pygame.sprite.groupcollide(self.sprEnemies, self.sprSwords, 0, 0):
            playerInfo.setScore(player.addScore(int(enemies.hit(200, self.sprEnemies)*1.5)))
        pygame.sprite.groupcollide(self.sprBullets, self.sprSwords, 1, 0)
        # take care of some updates & drawing
        self.sprSwords.update(self.sprSwords)
        self.sprSwords.draw(screen)
        self.sprEnemies.update(player, self.levelLights[player.level], self.floorLights[player.level], self.sprBullets)
        self.sprEnemies.draw(screen)
        self.sprEffects.update(self.sprEffects)
        self.sprEffects.draw(screen)
        self.sprBullets.update(self.sprEffects)
        self.sprBullets.draw(screen)
        self.sprExtra1.draw(screen)
        
        for enemies in pygame.sprite.spritecollide(player, self.sprEnemies, 0):
            playerInfo.setHealth(player.damage(30, True)) # 30 points of damage, and we have knockback
            break                   # only want to damage our hero once
        for bullets in pygame.sprite.spritecollide(player, self.sprBullets, 1):
            playerInfo.setHealth(player.damage(bullets.bulletDamage(), False))

        for extras in pygame.sprite.spritecollide(player, self.sprExtra1, 1):
            if extras.name() == "HealthPack":
                playerInfo.setHealth(player.addHealth(20))

    def complete(self):
        return self.levelComplete

    def nextLife(self):
        self.sprSwords.empty()
        self.sprEffects.empty()
        self.sprBullets.empty()
