#/usr/bin/env python
"""
Urban Ninja is a rockin game made for PyWeek3.
http://www.pyweek.org/3/

Coded by Cthulhu32 (Luke A.)
"""

#Import Modules
import os, pygame
from pygame.locals import *

#Import Local Modules
from player import Ninja
from level import Level
from userinterface import TopBar

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

#functions to create our resources
def load_image(name, colorkey=None):
    fullname = os.path.join('Data', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()
     
def load_sound(name):
    class NoneSound:
        def play(self): pass
    if not pygame.mixer or not pygame.mixer.get_init():
        return NoneSound()
    fullname = os.path.join('Music', name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
        print 'Cannot load sound:', fullname
        raise SystemExit, message
    return sound
   
def main():
    """T3h Main Functionz
        Word"""
#Initialize Everything
    pygame.init()
    screen = pygame.display.set_mode((640, 480))
    pygame.display.set_caption('Urban Ninja   -  {Pyweek3} Luke Arntson \'06')
    pygame.mouse.set_visible(0)

    highScore = 200

    while 1:

        menuState = 1 # ready to start
        splashScreen, splashRect = load_image('splashScreen.gif')
        screen.blit(splashScreen, (0,0))
        cursor, cursorRect = load_image('cursorImage.gif', (0,255,0))
        screen.blit(cursor, (240, 200))
        font = pygame.font.Font(None, 36)
        scoreText = font.render(str(highScore), 1, (0, 0, 0))
        screen.blit(scoreText, (125, 446))
        while 1:

        #Handle Input Events
            for event in pygame.event.get():
                if event.type == QUIT:
                    return
                elif event.type == KEYDOWN and event.key == K_ESCAPE:
                    return
                if event.type == KEYDOWN and event.key == K_RETURN:
                    if menuState == 0:
                        return
                    elif menuState == 1:
                        menuState = 2
                if event.type == KEYDOWN and (event.key == K_UP or event.key == K_DOWN):
                    if menuState == 0:
                        menuState = 1
                        screen.blit(splashScreen, (0,0))
                        cursor, cursorRect = load_image('cursorImage.gif', (0,255,0))
                        screen.blit(cursor, (240, 200))
                        scoreText = font.render(str(highScore), 1, (0, 0, 0))
                        screen.blit(scoreText, (125, 446))
                    elif menuState == 1:
                        menuState = 0
                        screen.blit(splashScreen, (0,0))
                        cursor, cursorRect = load_image('cursorImage.gif', (0,255,0))
                        screen.blit(cursor, (240, 290))
                        scoreText = font.render(str(highScore), 1, (0, 0, 0))
                        screen.blit(scoreText, (125, 446))
            if menuState == 2:
                loading, loadingRect = load_image('loadingScreen.gif')
                screen.blit(loading, (0,0))
                pygame.display.flip()
                break
            pygame.display.flip()



        gameMusic = load_sound('GO.ogg')
        gameMusic.set_volume(0.25)
        gameMusic.play(-1)

    #Create The Backgound
        background = pygame.Surface(screen.get_size())
        background = background.convert()
        background.fill((255, 255, 255))

        shadow = pygame.Surface(screen.get_size())
        shadow = shadow.convert()

    #Display The Background
        screen.blit(background, (0, 0))
        pygame.display.flip()

    #Prepare Game Objects
        clock = pygame.time.Clock()
        player1 = Ninja((50, 43))
        playerInfo = TopBar(0, 150, 100, 3)
        allsprites = pygame.sprite.RenderPlain((player1))
        infoGroup = pygame.sprite.RenderPlain((playerInfo))

        gameDone = 0

        for i in range(1,5):   # we have levels 1 and 4

            player1.levelReset((50,43))

            # setup our shadow
            shadow.fill((0, 0, 0))
            shadow.set_colorkey((0,255,0), RLEACCEL)
            shadow.set_alpha(180)
            # Level class
            currentLevel = Level('Level'+str(i)+'.txt', background, shadow)
            currentTime = 10
            playerInfo.setTime(currentTime)
            setTimer = 0
        #Main Loop
            while not currentLevel.complete():
                clock.tick(60)

                if setTimer > 59:
                    setTimer = 0
                    currentTime = currentTime - 1
                    if currentTime < 0:
                        player1.removeLife()
                        player1.levelReset((50,43))
                        currentLevel.nextLife()
                        playerInfo.setLives(player1.currentLife())
                        playerInfo.setHealth(100)
                        playerInfo.setWeapon(0)
                        currentTime = 150
                        playerInfo.setTime(currentTime)
                    else:
                        playerInfo.setTime(currentTime)
                else:
                    setTimer = setTimer + 1
                if player1.stillPlaying() == False:
                    # return to the Main Menu Screen
                    print "No more lives"
                    gameDone = 1

                if player1.deadLife() == True:
                    print "Onto your next life"
                    player1.levelReset((50,43))
                    currentLevel.nextLife()
                    playerInfo.setLives(player1.currentLife())
                    playerInfo.setHealth(100)
                    playerInfo.setWeapon(0)

            #Handle Input Events
                for event in pygame.event.get():
                    if event.type == QUIT:
                        gameDone = 1
                    elif event.type == KEYDOWN and event.key == K_ESCAPE:
                        gameDone = 1
                    if event.type == KEYDOWN and event.key == K_LEFT:
                        player1.moveLeft()
                    elif event.type == KEYDOWN and event.key == K_RIGHT:
                        player1.moveRight()
                    elif event.type == KEYDOWN and event.key == K_UP:
                        player1.checkAction(currentLevel.checkMap(player1, background, shadow))
                    elif event.type == KEYDOWN and event.key == K_DOWN:
                        player1.checkAction(currentLevel.checkMapDown(player1))
                    if event.type == KEYDOWN and event.key == 97: #  'a'
                        player1.aim()
                    if event.type == KEYDOWN and event.key == 115: # 's'
                        playerInfo.setWeapon(player1.switchWeapon())                    
                    if event.type == KEYDOWN and event.key == K_SPACE:
                        currentLevel.playerAttack(player1.attack(), player1, playerInfo)
                    elif event.type == KEYUP and event.key == K_SPACE:
                        player1.stopAttack()
                    if event.type == KEYUP and (event.key == K_LEFT or event.key == K_RIGHT):
                        player1.stop()
                allsprites.update()

            #Draw Everything
                screen.blit(background, (0, 0))
                allsprites.draw(screen)
                currentLevel.update(screen, player1, playerInfo)
                screen.blit(shadow, (0, 0))
                infoGroup.draw(screen)
                
                pygame.display.flip()

                if gameDone == 1:
                    gameMusic.stop()
                    break
        if playerInfo.score > highScore:
            highScore = playerInfo.score
            print " New High Score = " + str(highScore)
    #Game Over


#this calls the 'main' function when this script is executed
if __name__ == '__main__': main()
