import os, pygame
from pygame.locals import *

#functions to create our resources
def load_image(name, colorkey=None):
    fullname = os.path.join('Data', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()

class Bullet(pygame.sprite.Sprite):
    """ Ladder allows people to move around """
    def __init__(self, xy, speed, flip, bulletType):
        pygame.sprite.Sprite.__init__(self)
        bulletName = 'bullet' + str(bulletType) + '.gif'
        self.image, self.rect = load_image(bulletName)
        self.rect.topleft = xy
        self.facingRight = flip
        if self.facingRight == False: #We are not facing right
            self.image = pygame.transform.flip(self.image, 1, 0)
        self.speed = speed

    def update(self, spriteList):
        x,y = self.rect.topleft
        if self.facingRight == 1:
            x += self.speed
        else:
            x -= self.speed
        if (x < 0 or x > 640):
            spriteList.remove(self)
        self.rect.topleft = (x,y)

    def bulletDamage(self):
        return 20

    def name(self):
        return "Bullet"

class Flash(pygame.sprite.Sprite):
    """ Muzzle flash! """
    def __init__(self, xy, face):
        pygame.sprite.Sprite.__init__(self)
        self.image, self.rect = load_image('gun_muzzle1.gif', -1)
        self.muzzleImage    =   []
        muzzleI2, null = load_image('gun_muzzle2.gif', -1)
        muzzleI3, null = load_image('gun_muzzle3.gif', -1)
        muzzleI4, null = load_image('gun_muzzle4.gif', -1)
        muzzleI5, null = load_image('gun_muzzle5.gif', -1)
        self.muzzleImage.append(muzzleI2)
        self.muzzleImage.append(muzzleI3)
        self.muzzleImage.append(muzzleI4)
        self.muzzleImage.append(muzzleI5)
        self.muzzleImage.append(muzzleI5)

        self.muzzleImageRev =   []
        muzzleR2 = pygame.transform.flip(muzzleI2,1,0)
        muzzleR3 = pygame.transform.flip(muzzleI3,1,0)
        muzzleR4 = pygame.transform.flip(muzzleI4,1,0)
        muzzleR5 = pygame.transform.flip(muzzleI5,1,0)
        self.muzzleImageRev.append(muzzleR2)
        self.muzzleImageRev.append(muzzleR3)
        self.muzzleImageRev.append(muzzleR4)
        self.muzzleImageRev.append(muzzleR5)
        self.muzzleImageRev.append(muzzleR5)

        self.face = face
        if face == "shootLeft":
            x,y = xy
            x = x - 16
            y = y + 6
            self.rect.topleft = x,y
        elif face == "shootRight":
            x,y = xy
            x = x + 32
            y = y + 6
            self.rect.topleft = x,y
            self.image = pygame.transform.flip(self.image, 1, 0)
        self.life = 5
        
    def update(self, spriteList):
        if self.life > 0:
            self.life = self.life - 1
            if self.face == "shootLeft":
                self.image = self.muzzleImage[4-self.life]
            elif self.face == "shootRight":
                self.image = self.muzzleImageRev[4-self.life]
        else:
            spriteList.remove(self)

    def name(self):
        return "Flash"

class HitFlash(pygame.sprite.Sprite):
    """ Hit Flash! """
    def __init__(self, xy):
        pygame.sprite.Sprite.__init__(self)
        self.image, self.rect = load_image('gun_hit1.gif', -1)
        self.rect.center = xy
        self.life = 3
        
    def update(self, spriteList):
        if self.life > 0:
            self.life = self.life - 1
        else:
            spriteList.remove(self)

    def name(self):
        return "HitFlash"

class Sword(pygame.sprite.Sprite):
    """ Sword Slash! """
    def __init__(self, xy, faceLeft):
        pygame.sprite.Sprite.__init__(self)
        self.image, self.rect = load_image('sword_flash1.gif', -1)
        self.swordImage    =   []
        sword2, null = load_image('sword_flash2.gif', -1)
        sword3, null = load_image('sword_flash3.gif', -1)
        self.swordImage.append(self.image)
        self.swordImage.append(sword2)
        self.swordImage.append(sword3)

        self.swordImageRev =   []
        self.swordImageRev.append(pygame.transform.flip(self.image, 1, 0))
        self.swordImageRev.append(pygame.transform.flip(sword2, 1, 0))
        self.swordImageRev.append(pygame.transform.flip(sword3, 1, 0))
        
        self.life = 12
        self.face = faceLeft
        
        if self.face == "swordLeft":
            x,y = xy
            x = x - 15
            self.rect.topleft = x,y
        elif self.face == "swordRight":
            x,y = xy
            x = x + 15
            self.rect.topleft = x,y
            self.image = pygame.transform.flip(self.image, 1, 0)
        
    def update(self, spriteList):
        if self.life > 0:
            self.life = self.life - 1
            if self.face == "swordLeft":
                self.image = self.swordImage[2-self.life/4]
            elif self.face == "swordRight":
                self.image = self.swordImageRev[2-self.life/4]
        else:
            spriteList.remove(self)

    def name(self):
        return "Sword"
        
